package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.*;

import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.List;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 16:40
 */
public class UnshareLanTask extends Task {
    @Nonnull private final Slice slice;
    @Nonnull private final Server auth;
    @Nonnull private final String lanClientId;
    @Nonnull private final String sharedLanName;

    @Nonnull private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    @Nonnull private final HighLevelTaskFactory highLevelTaskFactory;

    public UnshareLanTask(@Nonnull Slice slice,
                          @Nonnull Server auth,
                          @Nonnull String lanClientId,
                          @Nonnull String sharedLanName,
                          @Nonnull AggregateManagerWrapperFactory aggregateManagerWrapperFactory,
                          @Nonnull HighLevelTaskFactory highLevelTaskFactory) {
        super("Unshare Lan " + lanClientId + " as " + sharedLanName);
        this.slice = slice;
        this.auth = auth;
        this.lanClientId = lanClientId;
        this.sharedLanName = sharedLanName;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.highLevelTaskFactory = highLevelTaskFactory;
    }

    @Override
    public void doTask(@Nonnull TaskExecution taskExecution) throws JFedException, InterruptedException {
        AggregateManagerWrapper amWrapper =
                aggregateManagerWrapperFactory.getAggregateManagerWrapper(taskExecution, auth);
        amWrapper.unshareLan(slice.getUrn(), slice.getCredentials(), lanClientId, sharedLanName);
    }

    @Nonnull
    @Override
    public List<Task> initDependsOn() {
        return Collections.singletonList(highLevelTaskFactory.getSliceCredential(slice));
    }
}
